package secure.counter.client;

import java.io.FileInputStream;
import javax.xml.rpc.Stub;

import org.apache.axis.message.MessageElement;
import org.apache.axis.message.addressing.EndpointReferenceType;

import org.globus.axis.util.Util;
import org.globus.wsrf.encoding.ObjectDeserializer;

import org.oasis.wsrf.properties.GetResourcePropertyResponse;

import org.xml.sax.InputSource;

import secure.counter.Counter;
import secure.counter.IncrementRequest;
import secure.counter.LastIncrementType;
import secure.counter.impl.CounterResource;
import secure.counter.service.CounterServiceAddressingLocator;

public class CounterAdd {
    static {
        Util.registerTransport();
    }    

    public static void main(String[] args) throws Exception {
        CounterServiceAddressingLocator locator =
            new CounterServiceAddressingLocator();
        
	try {
            String eprFile = args[0];
            if (args.length < 2) {
                throw new ArrayIndexOutOfBoundsException();
            }
            int[] values = new int[args.length - 1];
            for(int i = 1; i < args.length; i++) {
                values[i-1] = Integer.parseInt(args[i]);
            }

            // Get endpoint reference of WS-Resource from file
            FileInputStream fis = new FileInputStream(eprFile);
            EndpointReferenceType endpoint = (EndpointReferenceType)
                ObjectDeserializer.deserialize(
                    new InputSource(fis), EndpointReferenceType.class
                );
            fis.close();

            // Get counter service
            Counter counter = locator.getCounterPort(endpoint);

            // Configure security
            SecurityHelper.setSecurity((Stub) counter);            

            // Perform addition
            System.out.print("Adding values ...");
            counter.increment(new IncrementRequest(values));
            System.out.println(" done.");            

            // Access counter resource properties
            GetResourcePropertyResponse valueRP = counter.getResourceProperty(
                CounterResource.VALUE_RP
            );
            System.out.println("Value RP: " + valueRP.get_any()[0].getValue());

            GetResourcePropertyResponse historyRP = counter.getResourceProperty(
                CounterResource.HISTORY_RP
            );
            MessageElement[] historyElements = historyRP.get_any();
            System.out.print("History RP:");
            for(int i = 0; i < historyElements.length; i++) {
                System.out.print(" " + historyElements[i].getValue());
            }
            System.out.println("");

            GetResourcePropertyResponse lastIncrementRP = counter.getResourceProperty(
                CounterResource.LAST_INCREMENT_RP
            );
            LastIncrementType lastIncr = (LastIncrementType)
                ObjectDeserializer.getFirstAsObject(
                    lastIncrementRP, LastIncrementType.class
                );
            System.out.println("Last increment (" + lastIncr.getIncrement() +
                               ") occurred at: " + lastIncr.getTime().getTime());
            
	} catch (ArrayIndexOutOfBoundsException e) {
            System.err.println("Usage: CounterAdd <EPR file> values ... ");
	}
	    
    }
}
