package secure.counter.client;

import java.io.BufferedWriter;
import java.io.FileWriter;
import javax.xml.rpc.Stub;

import org.apache.axis.message.addressing.Address;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.globus.axis.util.Util;

import org.globus.wsrf.encoding.ObjectSerializer;

import secure.counter.Counter;
import secure.counter.CreateCounterRequest;
import secure.counter.CreateCounterResponse;
import secure.counter.service.CounterServiceAddressingLocator;

public class CounterCreate {
    static {
        Util.registerTransport();
    }
    
    public static void main(String[] args) throws Exception {
	try {
            String serviceURI = args[0];
            String eprFile = args[1];

            System.out.println("Contacting service " + serviceURI + " ...");
            
            CounterServiceAddressingLocator locator =
                new CounterServiceAddressingLocator();
            
            // Get counter service reference (no resource)
            EndpointReferenceType endpoint = new EndpointReferenceType();
            endpoint.setAddress(new Address(serviceURI));
            Counter counterService = locator.getCounterPort(endpoint);

            // Configure security
            SecurityHelper.setSecurity((Stub) counterService);            
            
            // Create a counter WS-resource
            CreateCounterResponse createResponse =
                counterService.createCounter(new CreateCounterRequest());
            endpoint = createResponse.getEndpointReference();

            // Store counter EPR in file
            String eprString = ObjectSerializer.toString(
                endpoint, endpoint.getTypeDesc().getXmlType()
            );
            
            FileWriter fileWriter = new FileWriter(eprFile);
            BufferedWriter bfWriter = new BufferedWriter(fileWriter);
            bfWriter.write(eprString + "\n");
            bfWriter.close();
            System.out.println("EPR written to file " + eprFile);
            
	} catch (ArrayIndexOutOfBoundsException e) {
            System.err.println("Usage: CounterCreate <service URI> <EPR output file>");
	}	    
    }
}
